"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const fs_1 = require("fs");
const axios_1 = __importDefault(require("axios"));
const https_1 = __importDefault(require("https"));
const sequelize_1 = require("sequelize");
/**
 * Once initialized this object contains `sequelize` instance as well as all DB models, keyed by model name.
 */
const db = {};
function wait(seconds) {
    return new Promise(resolve => setTimeout(resolve, 1000 * seconds));
}
function addHooks(sequelize, restart) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore afterDisconnect is static method
    sequelize.afterDisconnect(async (connection) => {
        const { _invalid: unexpectedDisconnection } = connection;
        if (unexpectedDisconnection) {
            restart('Unexpected disconnection occured.');
        }
    });
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore beforeQuery is not included in TypeScript declaration for sequelize,
    // See: https://github.com/sequelize/sequelize/issues/11441
    sequelize.beforeQuery(async ({ isRecoveryCheck }) => {
        if (!isRecoveryCheck) {
            const result = (await sequelize.query('SELECT pg_is_in_recovery();', {
                plain: true,
                type: sequelize_1.QueryTypes.SELECT,
                isRecoveryCheck: true
            }));
            if (result && result.pg_is_in_recovery) {
                restart('DB is in recovery.');
            }
        }
    });
}
/**
 * Constructs new object containing `init` function and `db` object
 *
 * @param {DbConfig} dbConfig DB configuration object
 * @param {LoggerFactory} loggerFactory object containing `getLogger` function
 * @param {ModelFactories} modelFactories array of factory functions returning sequelize model
 *
 * @returns {DbModule} DB module
 */
function getDbModule(dbConfig, loggerFactory, modelFactories) {
    const logger = loggerFactory.getLogger('DBConnection');
    function addModels(sequelize) {
        modelFactories.forEach(modelFactory => {
            const model = modelFactory(sequelize, sequelize_1.DataTypes);
            db[model.name] = model;
        });
    }
    function getDbOptions(configOptions) {
        const options = (0, lodash_1.merge)({
            logging: (message) => logger.debug(message)
        }, configOptions);
        if (options.dialectOptions?.ssl) {
            options.dialectOptions.ssl.ca = (0, fs_1.readFileSync)(options.dialectOptions.ssl.ca, { encoding: 'utf8' });
            if (options.dialectOptions.ssl.cert && options.dialectOptions.ssl.key) {
                // If the cert is provided, the key will also be provided by the installer.
                options.dialectOptions.ssl.cert = (0, fs_1.readFileSync)(options.dialectOptions.ssl.cert, {
                    encoding: 'utf8'
                });
                options.dialectOptions.ssl.key = (0, fs_1.readFileSync)(options.dialectOptions.ssl.key, { encoding: 'utf8' });
            }
        }
        return options;
    }
    async function selectDbUrl() {
        function getHostname(dbUrl) {
            return new URL(dbUrl).hostname;
        }
        function getCertificateAuthority() {
            const { dialectOptions } = dbConfig.options;
            return dialectOptions?.ssl ? (0, fs_1.readFileSync)(dialectOptions.ssl.ca, { encoding: 'utf8' }) : undefined;
        }
        function isResponding(url) {
            const patroniUrl = `https://${getHostname(url)}:8008`;
            const ca = getCertificateAuthority();
            const config = ca ? { httpsAgent: new https_1.default.Agent({ ca }) } : undefined;
            return (0, axios_1.default)(patroniUrl, config)
                .then(response => response.status === 200)
                .catch(error => {
                logger.error(`Error occured when requesting: ${patroniUrl}.`, error);
                return false;
            });
        }
        async function findRespondingHost(urls) {
            let respondingHost = null;
            /* eslint-disable no-await-in-loop */
            do {
                for (let i = 0; i < urls.length; i += 1) {
                    const hostname = getHostname(urls[i]);
                    logger.info(`Checking DB host ${i}: ${hostname}`);
                    const hasResponded = await isResponding(urls[i]);
                    if (hasResponded) {
                        logger.debug(`DB host ${hostname} has responded.`);
                        respondingHost = urls[i];
                        break;
                    }
                    else {
                        logger.debug(`DB host ${hostname} not responding.`);
                    }
                }
                if (!respondingHost) {
                    await wait(1);
                    logger.info(`Retrying DB host selection...`);
                }
            } while (!respondingHost);
            /* eslint-enable no-await-in-loop */
            return respondingHost;
        }
        const { url: dbUrls } = dbConfig;
        let selectedDbUrl = null;
        if (dbUrls && (0, lodash_1.isString)(dbUrls)) {
            selectedDbUrl = dbUrls;
        }
        else if ((0, lodash_1.isArray)(dbUrls)) {
            logger.info('Selecting DB host...');
            selectedDbUrl = await findRespondingHost(dbUrls);
        }
        else {
            throw new Error('Invalid db.url parameter passed to the configuration. Expected not empty string or array of strings.');
        }
        logger.info(`Selected DB host: ${getHostname(selectedDbUrl)}`);
        return selectedDbUrl;
    }
    async function connect(sequelize, restart) {
        try {
            await sequelize.authenticate();
            logger.info('DB connection has been established successfully.');
        }
        catch (error) {
            logger.error(error);
            restart('Unable to connect to the database.');
        }
    }
    /**
     * Initializes DB connection
     *
     * @returns {Promise<void>} promise resolving once connection is established
     */
    async function init() {
        const { options } = dbConfig;
        const dbOptions = getDbOptions(options);
        const dbUrl = await selectDbUrl();
        const sequelize = new sequelize_1.Sequelize(dbUrl, dbOptions);
        db.sequelize = sequelize;
        let isRestarting = false;
        async function restart(reason) {
            if (!isRestarting) {
                isRestarting = true;
                logger.info(reason);
                logger.info('Closing all DB connections...');
                await sequelize.close();
                logger.info('Re-initializing DB...');
                await wait(1);
                init();
            }
        }
        addModels(sequelize);
        addHooks(sequelize, restart);
        await connect(sequelize, restart);
    }
    return { init, db };
}
exports.default = getDbModule;
//# sourceMappingURL=db.js.map