"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const winston_1 = __importDefault(require("winston"));
const lodash_1 = require("lodash");
const fs_1 = require("fs");
const events_1 = __importDefault(require("events"));
function getArgsSupportedLogger(logger) {
    // This is workaround for no support for multi-arguments logging, e.g.: logger.info('Part 1', 'Part 2')
    // See: https://github.com/winstonjs/winston/issues/1614
    const wrapper = (original) => {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return (...args) => {
            for (let index = 0; index < args.length; index += 1) {
                if (args[index] instanceof Error) {
                    args[index] = args[index].stack;
                }
                else if (!(0, lodash_1.isString)(args[index])) {
                    try {
                        const stringifiedObject = JSON.stringify(args[index]);
                        args[index] = stringifiedObject;
                    }
                    catch (error) {
                        args[index] = '<NotParsable>';
                    }
                }
            }
            return original(args.join(' '));
        };
    };
    logger.error = wrapper(logger.error);
    logger.warn = wrapper(logger.warn);
    logger.info = wrapper(logger.info);
    logger.verbose = wrapper(logger.verbose);
    logger.debug = wrapper(logger.debug);
    logger.silly = wrapper(logger.silly);
    logger.log = logger.info;
    return logger;
}
/**
 * Initializes logging framework according to the given config
 *
 * @param {LoggerConfig} config configuration object
 * @param {number} defaultMaxListeners optional number of default event listeners to be assigned to
 * EventEmitter.defaultMaxListeners, should be set to at least the number of logging categories to be used, defaults to 30
 *
 * @returns {LoggerFactory} object containing `getLogger` and `logErrorsOnly` functions
 */
function initLogging(config, defaultMaxListeners = 30) {
    events_1.default.EventEmitter.defaultMaxListeners = defaultMaxListeners;
    /*
     * Reads log level from manager's configuration file, as specified by `logLevelConf` configuration parameter.
     * See https://github.com/cloudify-cosmo/cloudify-manager/blob/master/packaging/mgmtworker/files/etc/cloudify/logging.conf
     */
    function getLevelFromLoggingConf() {
        try {
            return (0, lodash_1.chain)((0, fs_1.readFileSync)(config.logLevelConf))
                .split('\n')
                .map(lodash_1.trim)
                .filter(fileEntry => !fileEntry.startsWith('#'))
                .map(fileEntry => fileEntry.split(/\s+/))
                .find(fileEntryWords => (0, lodash_1.last)(fileEntryWords) === config.serviceName)
                .first()
                .toLower()
                .thru(l => (l === 'warning' ? 'warn' : l))
                .value();
        }
        catch (e) {
            // eslint-disable-next-line no-console
            console.warn(`Couldn't read logging level from ${config.logLevelConf}:`, e);
            return null;
        }
    }
    const transports = [
        new winston_1.default.transports.Console({
            format: winston_1.default.format.colorize({ all: true })
        }),
        new winston_1.default.transports.File({ filename: config.logsFile }),
        new winston_1.default.transports.File({ filename: config.errorsFile, level: 'error' })
    ];
    const logFormat = winston_1.default.format.printf(({ level, message, label, timestamp }) => `[${timestamp}][${label}] ${(0, lodash_1.upperCase)(level)}: ${message}`);
    const level = (config.logLevelConf && getLevelFromLoggingConf()) || config.logLevel;
    /**
     * Returns logger for given category
     *
     * @param {string} category category
     *
     * @returns {Logger} enhanced winston logger instance
     */
    function getLogger(category) {
        if (winston_1.default.loggers.has(category)) {
            return winston_1.default.loggers.get(category);
        }
        const logger = winston_1.default.loggers.add(category, {
            level,
            transports,
            format: winston_1.default.format.combine(winston_1.default.format.label({ label: category }), winston_1.default.format.timestamp(), logFormat)
        });
        return getArgsSupportedLogger(logger);
    }
    /**
     * Sets threshold log level to `error`
     */
    function logErrorsOnly() {
        transports.forEach(transport => {
            transport.level = 'error';
        });
    }
    return {
        getLogger,
        logErrorsOnly
    };
}
exports.default = initLogging;
//# sourceMappingURL=logger.js.map